//  Playinfo.inc

constructor TPlayInfo.Create(aPanel:TPlayPanel; aHeight:integer; aIdx:integer; pInf:TPlayInfo);
begin
  inherited Create(aPanel);
  Parent := aPanel;
  FIndex := aIdx;
  FCont := TPlayContainer(aPanel.Parent);
  AutoSize := true;
  BevelInner := bvRaised;  // Kan være anderledes...
  BevelOuter := bvNone;
  Color := TPlayContainer(aPanel.Parent).ColorNorm;
  AnchorSide[akLeft].Side := asrLeft;
  AnchorSide[akLeft].Control := aPanel;
  if Assigned(pInf) then begin
    AnchorSide[akTop].Side := asrBottom;
    AnchorSide[akTop].Control := pInf;
  end else begin
    AnchorSide[akTop].Side := asrTop;
    AnchorSide[akTop].Control := aPanel;
  end;
  AnchorSide[akRight].Side := asrRight;
  AnchorSide[akRight].Control := aPanel;
  BorderSpacing.Around := 0;
  BorderSpacing.Left := 1;
  BorderSpacing.Top := 0;
  BorderSpacing.Right := 1;
  BorderSpacing.Bottom := 1;
  Anchors := [akLeft, akTop, akRight];
  DragMode := dmManual;
  DragKind := dkDrag;
  FImage := TPlayImage.Create(Self, aHeight);
  FInfoLabel := TPlayLabel.Create(Self, FImage);
end;

procedure TPlayInfo.SetInfo(aImg, aTxt:string);
begin
  FImage.Picture.LoadFromFile(aImg);
  FInfoLabel.Caption := aTxt;
  Refresh;
end;

procedure TPlayInfo.MouseEnter;
begin
  FCont.Hover := FIndex;
  if Assigned(FCont.OnMouseEnter) then FCont.OnMouseEnter(Self);
end;

procedure TPlayInfo.MouseLeave;
begin
  FCont.Hover := -1;
  if Assigned(FCont.OnMouseLeave) then FCont.OnMouseLeave(Self);
end;

procedure TPlayInfo.MouseDown(aButton:TMouseButton; aShift:TShiftState; aX,aY: Integer);
begin
  FCont.Selected := FIndex;
  FCont.FMouseIndex := FIndex;
  if Assigned(FCont.OnMouseDown) then FCont.OnMouseDown(Self, aButton, aShift, aX, aY)
end;

procedure TPlayInfo.MouseMove(aShift:TShiftState; aX,aY: Integer);
begin
  if Assigned(FCont.OnMouseMove) then FCont.OnMouseMove(Self, aShift, aX, aY)
end;

procedure TPlayInfo.MouseUp(aButton:TMouseButton; aShift:TShiftState; aX,aY:Integer);
begin
  FCont.Hover := FIndex;
  if Assigned(FCont.OnMouseUp) then FCont.OnMouseUp(Self, aButton, aShift, aX, aY)
end;


procedure TPlayInfo.DragOver(aSource:TObject; aX,aY:Integer; aState: TDragState; var Accept:Boolean);
begin
  if Assigned(FCont.OnDragOver) then begin
    FCont.DropIndex := FIndex;
    FCont.OnDragOver(FCont, aSource, aX, aY, aState, Accept);
    if not Accept or (aState = dsDragLeave) then FCont.DropIndex := -1; // Mangler muligvis i Image og label
  end;
end;

procedure TPlayInfo.DragDrop(aSource:TObject; aX,aY:Integer);
begin
  if Assigned(FCont.OnDragDrop) then FCont.OnDragDrop(FCont, aSource, aX, aY);
end;

destructor TPlayInfo.Destroy;
begin
  FImage.Free;
  FInfoLabel.Free;
  inherited Destroy;
end;

