unit playcontainer;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Forms, ExtCtrls, StdCtrls, Controls, Graphics, contnrs, MyStringFunc;

type
  TFetchEvent = procedure(aSender:TObject; aInfo:string) of object;
  TIndexEvent = procedure(aSender:TObject; aIndex:integer) of object;

  TPlayContainer = class;
  TPlayInfo = class;

  TPlayHeader = class(TLabel)
  private
    dX, dY : integer;
  public
    constructor Create(aCont:TPlayContainer; aScrW:integer); reintroduce;
    procedure DragOver(aSource:TObject; aX,aY:Integer; aState: TDragState; var Accept:Boolean); override;
  end;

  TPlayScroll = class(TScrollBar)
  private
    dX, dY : integer;
  public
    constructor Create(aCont:TPlayContainer; aScrW,aHigh:integer); reintroduce;
    procedure Change; override;
    procedure DragOver(aSource:TObject; aX,aY:Integer; aState: TDragState; var Accept:Boolean); override;
  end;

  TPlayPanel = class(TCustomPanel)
  private
    dX, dY : integer;
  public
    constructor Create(aCont:TPlayContainer; aLbl:TPlayHeader; aScr:TPlayScroll); reintroduce;
    procedure DragOver(aSource:TObject; aX,aY:Integer; aState: TDragState; var Accept:Boolean); override;
  end;

  TPlayImage = class(TImage)
  private
    FCont : TPlayContainer;
  public
    constructor Create(aInfo:TPlayInfo; aHeight:integer); reintroduce;
    procedure MouseEnter; override;
    procedure MouseLeave; override;
    procedure MouseDown(aButton:TMouseButton; aShift:TShiftState; aX,aY: Integer); override;
    procedure MouseMove(aShift:TShiftState; aX,aY: Integer); override;
    procedure MouseUp(aButton:TMouseButton; aShift:TShiftState; aX,aY:Integer); override;
    procedure DragOver(aSource:TObject; aX,aY:Integer; aState: TDragState; var Accept:Boolean); override;
    procedure DragDrop(aSource:TObject; aX,aY:Integer); override;
  end;

  TPlayLabel = class(TLabel)
  private
    FCont : TPlayContainer;
  public
    constructor Create(aInfo:TPlayInfo; aImg:TPlayImage); reintroduce;
    procedure MouseEnter; override;
    procedure MouseLeave; override;
    procedure MouseDown(aButton:TMouseButton; aShift:TShiftState; aX,aY: Integer); override;
    procedure MouseMove(aShift:TShiftState; aX,aY: Integer); override;
    procedure MouseUp(aButton:TMouseButton; aShift:TShiftState; aX,aY:Integer); override;
    procedure DragOver(aSource:TObject; aX,aY:Integer; aState: TDragState; var Accept:Boolean); override;
    procedure DragDrop(aSource:TObject; aX,aY:Integer); override;
  end;

  TPlayInfo = class(TCustomPanel)
  private
    FCont : TPlayContainer;
    FIndex : integer;
    FImage : TPlayImage;
    FInfoLabel : TLabel;
  public
    constructor Create(aPanel:TPlayPanel; aHeight:integer; aIdx:integer; pInf:TPlayInfo); reintroduce;
    procedure SetInfo(aImg,aTxt:string);
    procedure MouseEnter; override;
    procedure MouseLeave; override;
    procedure MouseDown(aButton:TMouseButton; aShift:TShiftState; aX,aY: Integer); override;
    procedure MouseMove(aShift:TShiftState; aX,aY: Integer); override;
    procedure MouseUp(aButton:TMouseButton; aShift:TShiftState; aX,aY:Integer); override;
    procedure DragOver(aSource:TObject; aX,aY:Integer; aState: TDragState; var Accept:Boolean); override;
    procedure DragDrop(aSource:TObject; aX,aY:Integer); override;
    destructor Destroy; override;
  public
    property ListIndex : integer read FIndex write FIndex;
  end;


  { TPlayContainer }

  TPlayContainer = class(TPanel)
  private
    FOnFetch : TFetchEvent;
    FOnHover : TNotifyEvent;
    FOnSelect : TNotifyEvent;
    FOnRightClick : TNotifyEvent;
    FOnLayoutChanged : TNotifyEvent;
    FDragReset : TNotifyEvent;
    FWasWidth : integer; // Used when resizing to disable calling changes if not needed
    FHeadLabel : TPlayHeader;
    FScroller : TPlayScroll;
    FDispPanel : TPlayPanel;
    FHeader : string;
    FHeadHeight : integer;
    FHeadColor : TColor;
    FScrollWidth : integer;
    FImgHeight : Integer;
    FDisplayNumber : integer;
    FDispList : TFPObjectList;
    FInfoList : TStringList;
    Xpos : integer; // Resizing
    FHover : integer;    // FHover og FSelected er indexes i FInfoList
    FSelected : integer;
    FColNorm : TColor;
    FColHover : TColor;
    FColSel : TColor;
    FColDrop : TColor;
    FDropIndex : integer;
    FMouseIndex : integer;
    FSizing : boolean;
    procedure SetScrollWidth(aWidth:integer);
    procedure SetHeadHeight(aHeight:integer);
    procedure SetHeader(aHead:string);
    procedure SetImgHeight(aHeight:integer);
    procedure SetScroller;
    procedure ShowData;
    procedure SizeChanged(aSender:TObject);
    procedure CalculateDisplay;
    function GetHover : Integer;
    procedure SetItemColor(aIndex:integer; aCol:TColor);
    procedure SetHover(aIndex:integer);
    procedure SetSelected(aIndex:integer);
    procedure Select(aIndex:integer);
    procedure SetDropIndex(aIndex:integer);
  protected
    procedure Loaded; override;
    procedure Resize; override;
    procedure MouseEnter; override;
    procedure MouseLeave; override;
    procedure MouseDown(aButton:TMouseButton; aShift:TShiftState; aX,aY:Integer); override;
    procedure MouseMove(aShift:TShiftState; aX,aY:Integer); override;
    procedure MouseUp(aButton:TMouseButton; aShift:TShiftState; aX,aY:Integer); override;
    function DoMouseWheel(Shift:TShiftState; WheelDelta:Integer; MousePos:TPoint) : boolean; override;
    procedure RightClick(aIndex:integer);
  public
    constructor Create(aOwner:TComponent); override;
    procedure ClearData;
    procedure SetInfo(aInfo:TStringList);
    procedure DragOverScroll(aDelta : integer);
    procedure DragOver(aSource:TObject; aX,aY:Integer; aState: TDragState; var Accept:Boolean); override;
    destructor Destroy; override;
    property MouseIndex : integer read FMouseIndex write FMouseIndex default -1; // Used to make sure the dragged index is the selected index
    property DropIndex : integer read FDropIndex write SetDropIndex default -1;  // Index of Item dragged or dropped over
  published
    property Header : string read FHeader write SetHeader;
    property HeaderHeight : integer read FHeadHeight write SetHeadHeight;
    property HeaderColor : TColor read FHeadColor write FHeadColor;
    property HeaderLabel : TPlayHeader read FHeadLabel;
    property ScrollWidth : integer read FScrollWidth write SetScrollWidth;
    property ImgHeight : integer read FImgHeight write SetImgHeight;
    property ColorNorm : TColor read FColNorm write FColNorm default clWhite;
    property ColorHover : TColor read FColHover write FColHover default cl3DLight;
    property ColorSelect : TColor read FColSel write FColSel default clActiveCaption;
    property ColorDrop : TColor read FColDrop write FColDrop default $0000C2CE;
    property Selected : integer read FSelected write SetSelected;
    property Hover : integer read GetHover write SetHover;
    property OnFetch : TFetchEvent read FOnFetch write FOnFetch;
    property OnHoverChange : TNotifyEvent read FOnHover write FOnHover;
    property OnSelectedChange : TNotifyEvent read FOnSelect write FOnSelect;
    property OnLayoutChanged : TNotifyEvent read FOnLayoutChanged write FOnLayoutChanged;
    property OnRightClick : TNotifyEvent read FOnRightClick write FOnRightClick;
    property OnDragReset : TNotifyEvent read FDragReset write FDragReset;
  end;

implementation

{$I playpanel.inc}
{$i playimage.inc}
{$i playlabel.inc}
{$I playinfo.inc}

constructor TPlayContainer.Create(aOwner:TComponent);
begin
  inherited Create(aOwner);
  Caption := '';
  FSizing := false;
  FHeader := 'Header';
  FHeadHeight := 20;
  FHeadColor := clActiveBorder;
  FScrollWidth := 14;
  FImgHeight := 120;
  FInfoList := TStringList.Create;
  FDispList := TFPObjectList.Create;
  OnResize := @SizeChanged;
  FColNorm := clWhite;
  FColHover := cl3DLight;
  FColSel := clActiveCaption;
  FColDrop := $0000C2CE;
  FHeadLabel := TPlayHeader.Create(Self, FScrollWidth);
  FScroller := TPlayScroll.Create(Self, FScrollWidth, FHeadHeight);
  FDispPanel := TPlayPanel.Create(Self, FHeadLabel, FScroller);
  FHover := -1;
  FSelected := -1;
  FWasWidth := Width;
  FDropIndex := -1;
  FMouseIndex := -1;
  DragMode := dmManual;
  DragKind := dkDrag;
  CalculateDisplay;
  ShowData;
end;

procedure TPlayContainer.Loaded;
begin
  inherited Loaded;
  if Parent <> nil then begin
    Font := Parent.Font;
    HeaderHeight := Canvas.TextHeight('Xq') + 4;
  end;
  FScroller.Name := Name+'_Scrol';
  FHeadLabel.Name := Name+'_Header';
  FDispPanel.Name := Name+'_Disp';
  FHeadLabel.OnEndDrag := OnEndDrag;
end;

procedure TPlayContainer.Resize;
begin
  inherited;
end;

procedure TPlayContainer.SizeChanged(aSender:Tobject);
begin
  CalculateDisplay;
end;

procedure TPlayContainer.SetScrollWidth(aWidth:integer);
begin
  FScrollWidth := aWidth;
  if FScroller <> nil then FScroller.Width := aWidth;
  if Assigned(FHeadLabel) then FHeadLabel.BorderSpacing.Right := aWidth+2; // Der er 1 på hverside i Borderspacing
end;

procedure TPlayContainer.SetHeadHeight(aHeight:integer);
begin
  FHeadHeight := aHeight;
  if Assigned(FHeadLabel) then FHeadLabel.Height := aHeight;
  Refresh;
end;

procedure TPlayContainer.SetHeader(aHead:string);
begin
  FHeader := aHead;
  if Assigned(FHeadLabel) then FHeadLabel.Caption := aHead;
end;

procedure TPlayContainer.SetImgHeight(aHeight:integer);
begin
  FImgHeight := aHeight;
  CalculateDisplay;
end;

procedure TPlayContainer.CalculateDisplay;
var
  nr, idx : integer;
  aInf : TPlayInfo;
  wInf : TPlayInfo = nil;
begin
  nr := Trunc((FDispPanel.Height-4)/(FImgHeight+5));
  if (nr > 0) and (nr <> FDisplayNumber) then begin
    FDisplayNumber := nr;
    while FDispPanel.ControlCount > 0 do
      FDispList.Remove(FDispList[0]);
    for idx := 0 to FDisplayNumber-1 do begin
      aInf := TPlayInfo.Create(FDispPanel, FImgHeight, idx+FScroller.Position, wInf);
      FDispList.Add(aInf);
      wInf := aInf;
      DragKind := dkDrag;
      DragMode := dmManual;
    end;
    SetScroller;
  end;
end;

procedure TPlayContainer.SetScroller;
var
  tMax : integer = 0;
begin
  with FScroller do begin
    if FInfoList.Count > FDisplayNumber then tMax := FInfoList.Count - FDisplayNumber;
    if tMax <> Max then Max := tMax;
    SmallChange := 1;
    if FInfoList.Count > 2 * FDisplayNumber then LargeChange := FDisplayNumber-1
    else LargeChange := 1;
    Enabled := (Max > Min);
    // Position?
  end;
  ShowData;
end;

procedure TPlayContainer.ClearData;
begin
  SetInfo(nil);
end;

procedure TPlayContainer.SetInfo(aInfo:TStringList);
var
  idx : integer;
begin
  FInfoList.Free;
  FHover := -1;
  FSelected := -1;
  FInfoList := TStringList.Create;
  if Assigned(aInfo) then for idx := 0 to aInfo.Count-1 do
    FInfoList.Add(aInfo[idx]);
  CalculateDisplay;
  SetScroller;
  ShowData;
end;

procedure TPlayContainer.ShowData;
var
  idx : integer = 0;
  st : integer;
  aDisp : TPlayInfo;
begin
  if Assigned(FOnFetch) then begin
    st := FScroller.Position;
    while idx < FDispList.Count do begin
      aDisp := TPlayInfo(FDispList[idx]);
      if idx+st < FInfoList.Count then begin
        aDisp.ListIndex := idx+st;
        FOnFetch(aDisp, FInfoList[idx+st]);
        if idx+st = FSelected then aDisp.Color := FColSel
        else if idx = FHover then aDisp.Color := FColHover
        else aDisp.Color := FColNorm;
        aDisp.Show;
      end else aDisp.Hide;
      idx += 1;
    end;
  end;
end;

procedure TPlayContainer.MouseEnter;
begin // Fjern hover markering, containeren er det der omgiver elementerne, så der er ngen hover på elementer
  FHover := -1;
  if Assigned(OnMouseEnter) then OnMouseEnter(Self);
  if Assigned(FDragReset) then FDragReset(Self);
end;

procedure TPlayContainer.MouseLeave;
begin // Fjern hover markering, containeren er det der omgiver elementerne, så der er ngen hover på elementer
  FHover := -1;
  if Assigned(OnMouseLeave) then OnMouseLeave(Self);
  if Assigned(FDragReset) then FDragReset(Self);
end;

procedure TPlayContainer.MouseDown(aButton: TMouseButton; aShift: TShiftState; aX, aY: Integer);
begin
  if (aButton = mbLeft) and (aY < FHeadHeight) and (aX > FScroller.Left) then begin
    FSizing := true;
    Cursor := crSizeWE;
    XPos := aX;
    FWasWidth := Width;
  end;
  if Assigned(OnMouseDown) then OnMouseDown(self, aButton, aShift, aX, aY);
end;

procedure TPlayContainer.MouseMove(aShift: TShiftState; aX, aY: Integer);
begin
  if (aY < FHeadHeight) and (aX > FScroller.Left) then Cursor := crSizeWE;
  if FSizing then begin
    Width := Width + aX - XPos;
    XPos := aX;
  end else
    if Assigned(OnMouseMove) then OnMouseMove(self, aShift, aX, aY);
end;

procedure TPlayContainer.MouseUp(aButton: TMouseButton; aShift: TShiftState; aX, aY: Integer);
begin
  FSizing := false;
  Cursor := crDefault;
  if Assigned(FOnLayoutChanged) and (FWasWidth <> Width) then FOnLayoutChanged(self);
  if Assigned(FDragReset) then FDragReset(Self);
end;

function TPlayContainer.DoMouseWheel(Shift:TShiftState; WheelDelta:Integer; MousePos:TPoint) : boolean;
begin
  if (WheelDelta > 0) and (FScroller.Position > FScroller.Min) then
    FScroller.Position := FScroller.Position - 1;
  if (WheelDelta < 0) and (FScroller.Position < FScroller.Max) then
    FScroller.Position := FScroller.Position + 1;
 DoMouseWheel := true;
end;

procedure TPlayContainer.SetItemColor(aIndex:integer; aCol:TColor);
var
  dIndex : integer; // Index af item der skal ændre farve, i listen af viste items
begin
  dIndex := aIndex - FScroller.Position;
  if (dIndex > -1) and (dIndex < FDisplayNumber) then
    TPlayInfo(FDispList[dIndex]).Color := aCol;
end;

procedure TPlayContainer.SetHover(aIndex:integer);
var
  hLin : integer;
begin
  if aIndex > -1 then hLin := aIndex - FScroller.Position
  else hLin := -1;
  if (hLin <> FHover) and ((FSelected = -1) or (hLin <> FSelected)) then begin
    if (FHover <> FSelected) then SetItemColor(GetHover, FColNorm);
    FHover := hLin;
    SetItemColor(GetHover, FColHover);
  end else if (aIndex = FSelected) then FHover := -1;
  if Assigned(FOnHover) then FOnHover(self);
end;

function TPlayContainer.GetHover : Integer;
begin
  if FHover > -1 then GetHover := FHover + FScroller.Position
  else GetHover := -1;
end;

procedure TPlayContainer.SetSelected(aIndex:integer);
var  // Flytter scroll så selectede er vist - kalder select for faktisk at selected
  aPos : integer;
begin
  if (aIndex > -1) and (aIndex < FInfoList.Count) then begin
    if (aIndex < FScroller.Position) or (aIndex > FScroller.Position + FDispList.Count - 1) then begin
      aPos := aIndex - Round(FDisplayNumber/2);
      if apos < FScroller.Min then aPos := FScroller.Min;
      if apos > FScroller.Max then aPos := FScroller.Max;
      FScroller.Position := aPos;
    end;
    Select(aIndex);
  end else begin
    if FSelected > -1 then SetItemColor(FSelected, FColNorm);
    FSelected := -1;
  end;
end;

procedure TPlayContainer.DragOverScroll(aDelta : integer);
var
  nPos : integer;
begin
  nPos := FScroller.Position + aDelta;
  if nPos < FScroller.Min then nPos := FScroller.Min
  else if nPos > FScroller.Max then nPos := FScroller.Max;
  FScroller.Position := nPos;
end;

procedure TPlayContainer.DragOver(aSource:TObject; aX,aY:Integer; aState: TDragState; var Accept:Boolean);
begin
  Accept := false;
end;

procedure TPlayContainer.SetDropIndex(aIndex:integer);
var  // Flytter scroll så selectede er vist - kalder select for faktisk at selected
  aCol : TColor;
begin
  if FDropIndex > -1 then begin
    if FDropIndex <> FSelected then aCol := FColNorm
    else aCol := FColSel;
    SetItemColor(FDropIndex, aCol);
  end;
  FDropIndex := aIndex;
  SetItemColor(FDropIndex, FColDrop);
end;

procedure TPlayContainer.Select(aIndex:integer);
var
  wasSel : integer;
begin
  wasSel := FSelected;
  if aIndex <> wasSel then begin
    SetItemColor(aIndex, FColSel);
    if wasSel > -1 then SetItemColor(wasSel, FColNorm);
    FSelected := aIndex;
    if FSelected = FHover then FHover := -1;
  end else begin    // Der er klikket på den valgte - det skal svare til at fjerne valget
    SetItemColor(FSelected, FColHover);
    FSelected := -1;
    if aIndex > -1 then begin
      FHover := aIndex;
      SetItemColor(FHover, FColHover);
    end;
  end;
  if WasSel <> FSelected then
    if Assigned(FOnSelect) then FOnSelect(Self);
end;

procedure TPlayContainer.RightClick(aIndex:integer);
begin
  if aIndex <> FSelected then SetSelected(aIndex);
  if Assigned(FOnRightClick) then FOnRightClick(self);
end;

destructor TPlayContainer.Destroy;
begin
  FInfoList.Free;
  FHeadLabel.Free;
  FScroller.Free;
  FDispList.Free;
  FDispPanel.Free;
  inherited Destroy;
end;

end.

